import logging
from typing import Optional

from dateutil.parser import parse
from django.utils import timezone
from music.constants import JELLYFIN_POST_KEYS
from music.models import Track
from podcasts.models import Episode
from scrobbles.models import Scrobble
from scrobbles.utils import convert_to_seconds, parse_mopidy_uri
from videos.models import Video
from sports.models import SportEvent
from vrobbler.apps.music.utils import (
    get_or_create_album,
    get_or_create_artist,
    get_or_create_track,
)

logger = logging.getLogger(__name__)


def mopidy_scrobble_podcast(
    data_dict: dict, user_id: Optional[int]
) -> Scrobble:
    mopidy_uri = data_dict.get("mopidy_uri", "")
    parsed_data = parse_mopidy_uri(mopidy_uri)

    producer_dict = {"name": data_dict.get("artist")}

    podcast_name = data_dict.get("album")
    if not podcast_name:
        podcast_name = parsed_data.get("podcast_name")
    podcast_dict = {"name": podcast_name}

    episode_name = parsed_data.get("episode_filename")
    episode_dict = {
        "title": episode_name,
        "run_time_ticks": data_dict.get("run_time_ticks"),
        "run_time": data_dict.get("run_time"),
        "number": parsed_data.get("episode_num"),
        "pub_date": parsed_data.get("pub_date"),
        "mopidy_uri": mopidy_uri,
    }

    episode = Episode.find_or_create(podcast_dict, producer_dict, episode_dict)

    # Now we run off a scrobble
    mopidy_data = {
        "user_id": user_id,
        "timestamp": timezone.now(),
        "playback_position_ticks": data_dict.get("playback_time_ticks"),
        "source": "Mopidy",
        "mopidy_status": data_dict.get("status"),
    }

    scrobble = None
    if episode:
        scrobble = Scrobble.create_or_update(episode, user_id, mopidy_data)
    return scrobble


def mopidy_scrobble_track(
    data_dict: dict, user_id: Optional[int]
) -> Optional[Scrobble]:
    artist = get_or_create_artist(
        data_dict.get("artist"),
        mbid=data_dict.get("musicbrainz_artist_id", None),
    )
    album = get_or_create_album(
        data_dict.get("album"),
        artist=artist,
        mbid=data_dict.get("musicbrainz_album_id"),
    )
    track = get_or_create_track(
        title=data_dict.get("name"),
        mbid=data_dict.get("musicbrainz_track_id"),
        artist=artist,
        album=album,
        run_time_ticks=data_dict.get("run_time_ticks"),
        run_time=data_dict.get("run_time"),
    )

    # Now we run off a scrobble
    mopidy_data = {
        "user_id": user_id,
        "timestamp": timezone.now(),
        "playback_position_ticks": data_dict.get("playback_time_ticks"),
        "source": "Mopidy",
        "mopidy_status": data_dict.get("status"),
    }

    scrobble = Scrobble.create_or_update(track, user_id, mopidy_data)

    return scrobble


def build_scrobble_dict(data_dict: dict, user_id: int) -> dict:
    jellyfin_status = "resumed"
    if data_dict.get("IsPaused"):
        jellyfin_status = "paused"
    elif data_dict.get("NotificationType") == 'PlaybackStop':
        jellyfin_status = "stopped"

    playback_ticks = data_dict.get("PlaybackPositionTicks", "")
    if playback_ticks:
        playback_ticks = playback_ticks // 10000

    return {
        "user_id": user_id,
        "timestamp": parse(data_dict.get("UtcTimestamp")),
        "playback_position_ticks": playback_ticks,
        "playback_position": data_dict.get("PlaybackPosition", ""),
        "source": data_dict.get("ClientName", "Vrobbler"),
        "source_id": data_dict.get('MediaSourceId'),
        "jellyfin_status": jellyfin_status,
    }


def jellyfin_scrobble_track(
    data_dict: dict, user_id: Optional[int]
) -> Optional[Scrobble]:

    null_position_on_progress = (
        data_dict.get("PlaybackPosition") == "00:00:00"
        and data_dict.get("NotificationType") == "PlaybackProgress"
    )

    # Jellyfin has some race conditions with it's webhooks, these hacks fix some of them
    if null_position_on_progress:
        logger.error("No playback position tick from Jellyfin, aborting")
        return

    artist = get_or_create_artist(
        data_dict.get(JELLYFIN_POST_KEYS["ARTIST_NAME"]),
        mbid=data_dict.get(JELLYFIN_POST_KEYS["ARTIST_MB_ID"]),
    )
    album = get_or_create_album(
        data_dict.get(JELLYFIN_POST_KEYS["ALBUM_NAME"]),
        artist=artist,
        mbid=data_dict.get(JELLYFIN_POST_KEYS['ALBUM_MB_ID']),
    )

    run_time_ticks = (
        data_dict.get(JELLYFIN_POST_KEYS["RUN_TIME_TICKS"]) // 10000
    )
    run_time = convert_to_seconds(
        data_dict.get(JELLYFIN_POST_KEYS["RUN_TIME"])
    )
    track = get_or_create_track(
        title=data_dict.get("Name"),
        artist=artist,
        album=album,
        run_time_ticks=run_time_ticks,
        run_time=run_time,
    )

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    # A hack to make Jellyfin work more like Mopidy for music tracks
    scrobble_dict["playback_position_ticks"] = 0
    scrobble_dict["playback_position"] = ""

    return Scrobble.create_or_update(track, user_id, scrobble_dict)


def jellyfin_scrobble_video(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_imdb", None):
        logger.error(
            "No IMDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    video = Video.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(video, user_id, scrobble_dict)


def manual_scrobble_video(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_imdb", None):
        logger.error(
            "No IMDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    video = Video.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(video, user_id, scrobble_dict)


def manual_scrobble_event(data_dict: dict, user_id: Optional[int]):
    if not data_dict.get("Provider_thesportsdb", None):
        logger.error(
            "No TheSportsDB ID received. This is likely because all metadata is bad, not scrobbling"
        )
        return
    event = SportEvent.find_or_create(data_dict)

    scrobble_dict = build_scrobble_dict(data_dict, user_id)

    return Scrobble.create_or_update(event, user_id, scrobble_dict)



# Generated by Django 4.1.5 on 2023-01-14 21:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='sportevent',
            old_name='start_utc',
            new_name='start',
        ),
    ]



import logging
from django.utils import timezone

from imdb import Cinemagoer

imdb_client = Cinemagoer()

logger = logging.getLogger(__name__)


def lookup_video_from_imdb(imdb_id: str) -> dict:

    if 'tt' not in imdb_id:
        logger.warning(f"IMDB ID should begin with 'tt' {imdb_id}")
        return

    lookup_id = imdb_id.strip('tt')
    media = imdb_client.get_movie(lookup_id)

    run_time_seconds = 60 * 60
    runtimes = media.get("runtimes")
    if runtimes:
        run_time_seconds = int(runtimes[0]) * 60

    # Ticks otherwise known as miliseconds
    run_time_ticks = run_time_seconds * 1000 * 1000

    item_type = "Movie"
    if media.get('series title'):
        item_type = "Episode"

    try:
        plot = media.get('plot')[0]
    except TypeError:
        plot = ""
    except IndexError:
        plot = ""

    logger.debug(f"Received data from IMDB: {media.__dict__}")
    # Build a rough approximation of a Jellyfin data response
    data_dict = {
        "ItemType": item_type,
        "Name": media.get('title'),
        "Overview": plot,
        "Tagline": media.get('tagline'),
        "Year": media.get('year'),
        "Provider_imdb": imdb_id,
        "RunTime": run_time_seconds,
        "RunTimeTicks": run_time_ticks,
        "SeriesName": media.get('series title'),
        "EpisodeNumber": media.get('episode'),
        "SeasonNumber": media.get('season'),
        "PlaybackPositionTicks": 1,
        "PlaybackPosition": 1,
        "UtcTimestamp": timezone.now().strftime('%Y-%m-%d %H:%M:%S.%f%z'),
        "IsPaused": False,
        "PlayedToCompletion": False,
    }
    logger.debug(f"Parsed data from IMDB data: {data_dict}")

    return data_dict



# Generated by Django 4.1.5 on 2023-01-12 18:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('podcasts', '0002_episode_run_time_episode_run_time_ticks_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='episode',
            name='pub_date',
            field=models.DateField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-20 18:40

from uuid import uuid4
from django.db import migrations, models


def generate_uuids(apps, schema_editor):
    """Force uuid generation for old scrobbles"""
    Scrobble = apps.get_model('scrobbles', 'Scrobble')
    for scrobble in Scrobble.objects.all():
        if not scrobble.uuid:
            scrobble.uuid = uuid4()
            scrobble.save(update_fields=['uuid'])


def reverse_generate_uuids(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0008_scrobble_sport_event'),
    ]

    operations = [
        migrations.AddField(
            model_name='scrobble',
            name='uuid',
            field=models.UUIDField(blank=True, editable=False, null=True),
        ),
        migrations.RunPython(
            code=generate_uuids, reverse_code=reverse_generate_uuids
        ),
    ]



import logging
from datetime import datetime
import sqlite3
from enum import Enum

import pytz

from books.models import Author, Book
from scrobbles.models import Scrobble
from django.utils import timezone

logger = logging.getLogger(__name__)


class KoReaderBookColumn(Enum):
    ID = 0
    TITLE = 1
    AUTHORS = 2
    NOTES = 3
    LAST_OPEN = 4
    HIGHLIGHTS = 5
    PAGES = 6
    SERIES = 7
    LANGUAGE = 8
    MD5 = 9
    TOTAL_READ_TIME = 10
    TOTAL_READ_PAGES = 11


class KoReaderPageStatColumn(Enum):
    ID_BOOK = 0
    PAGE = 1
    START_TIME = 2
    DURATION = 3
    TOTAL_PAGES = 4


def process_koreader_sqlite_file(sqlite_file_path, user_id):
    """Given a sqlite file from KoReader, open the book table, iterate
    over rows creating scrobbles from each book found"""
    # Create a SQL connection to our SQLite database
    con = sqlite3.connect(sqlite_file_path)
    cur = con.cursor()

    # Return all results of query
    book_table = cur.execute("SELECT * FROM book")
    new_scrobbles = []
    for book_row in book_table:
        authors = book_row[KoReaderBookColumn.AUTHORS.value].split('\n')
        author_list = []
        for author_str in authors:
            logger.debug(f"Looking up author {author_str}")

            if author_str == "N/A":
                continue

            author, created = Author.objects.get_or_create(name=author_str)
            if created:
                author.fix_metadata()
            author_list.append(author)
            logger.debug(f"Found author {author}, created: {created}")

        book, created = Book.objects.get_or_create(
            koreader_md5=book_row[KoReaderBookColumn.MD5.value]
        )

        if created:
            book.title = book_row[KoReaderBookColumn.TITLE.value]
            book.pages = book_row[KoReaderBookColumn.PAGES.value]
            book.koreader_id = int(book_row[KoReaderBookColumn.ID.value])
            book.koreader_authors = book_row[KoReaderBookColumn.AUTHORS.value]
            book.run_time_ticks = int(book_row[KoReaderBookColumn.PAGES.value])
            book.save(
                update_fields=[
                    "title",
                    "pages",
                    "koreader_id",
                    "koreader_authors",
                ]
            )
            book.fix_metadata()
            if author_list:
                book.authors.add(*[a.id for a in author_list])

        playback_position = int(
            book_row[KoReaderBookColumn.TOTAL_READ_TIME.value]
        )
        playback_position_ticks = playback_position * 1000
        pages_read = int(book_row[KoReaderBookColumn.TOTAL_READ_PAGES.value])
        timestamp = datetime.utcfromtimestamp(
            book_row[KoReaderBookColumn.LAST_OPEN.value]
        ).replace(tzinfo=pytz.utc)

        new_scrobble = Scrobble(
            book_id=book.id,
            user_id=user_id,
            source="KOReader",
            timestamp=timestamp,
            playback_position_ticks=playback_position_ticks,
            playback_position=playback_position,
            played_to_completion=True,
            in_progress=False,
            book_pages_read=pages_read,
        )

        existing = Scrobble.objects.filter(
            timestamp=timestamp, book=book
        ).first()
        if existing:
            logger.debug(f"Skipping existing scrobble {new_scrobble}")
            continue

        logger.debug(f"Queued scrobble {new_scrobble} for creation")
        new_scrobbles.append(new_scrobble)

    # Be sure to close the connection
    con.close()

    created = Scrobble.objects.bulk_create(new_scrobbles)
    logger.info(
        f"Created {len(created)} scrobbles",
        extra={'created_scrobbles': created},
    )
    return created



# Generated by Django 4.1.5 on 2023-01-12 18:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('podcasts', '0003_episode_pub_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='episode',
            name='number',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



from django.contrib import admin
from scrobbles.models import Scrobble
from videos.models import Series, Video
from scrobbles.admin import ScrobbleInline


@admin.register(Series)
class SeriesAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name", "tagline")
    ordering = ("-created",)


@admin.register(Video)
class VideoAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    raw_id_fields = ('tv_series',)
    list_display = (
        "title",
        "video_type",
        "year",
        "tv_series",
        "season_number",
        "episode_number",
        "imdb_id",
    )
    list_filter = ("year", "tv_series", "video_type")
    ordering = ("-created",)
    inlines = [
        ScrobbleInline,
    ]



# Generated by Django 4.1.5 on 2023-02-03 22:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0013_audioscrobblertsvimport_processed_on'),
    ]

    operations = [
        migrations.AddField(
            model_name='audioscrobblertsvimport',
            name='process_log',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.contrib.auth import get_user_model
from rest_framework import serializers

User = get_user_model()

from profiles.models import UserProfile


class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        exclude = ('password',)


class UserProfileSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = UserProfile
        exclude = ('lastfm_password',)



# Generated by Django 4.1.5 on 2023-01-19 20:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music', '0008_alter_track_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='track',
            name='musicbrainz_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterUniqueTogether(
            name='track',
            unique_together={('album', 'musicbrainz_id')},
        ),
    ]



# Generated by Django 4.1.5 on 2023-02-20 00:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0021_scrobble_book'),
    ]

    operations = [
        migrations.AddField(
            model_name='scrobble',
            name='book_pages_read',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



import logging
import re

from musicbrainzngs.caa import musicbrainz

from scrobbles.musicbrainz import (
    lookup_album_dict_from_mb,
    lookup_artist_from_mb,
    lookup_track_from_mb,
)

logger = logging.getLogger(__name__)


from music.models import Album, Artist, Track


def get_or_create_artist(name: str, mbid: str = None) -> Artist:
    artist = None
    logger.debug(f'Got artist {name} and mbid: {mbid}')

    if 'feat.' in name.lower():
        name = re.split("feat.", name, flags=re.IGNORECASE)[0].strip()
    if 'featuring' in name.lower():
        name = re.split("featuring", name, flags=re.IGNORECASE)[0].strip()
    if '&' in name.lower():
        name = re.split("&", name, flags=re.IGNORECASE)[0].strip()

    artist_dict = lookup_artist_from_mb(name)
    mbid = mbid or artist_dict['id']

    logger.debug(f'Looking up artist {name} and mbid: {mbid}')
    artist = Artist.objects.filter(musicbrainz_id=mbid).first()
    if not artist:
        artist = Artist.objects.create(name=name, musicbrainz_id=mbid)
        logger.debug(
            f"Created artist {artist.name} ({artist.musicbrainz_id}) "
        )
        artist.fix_metadata()

    return artist


def get_or_create_album(name: str, artist: Artist, mbid: str = None) -> Album:
    album = None
    album_created = False
    albums = Album.objects.filter(name__iexact=name)
    if albums.count() == 1:
        album = albums.first()
    else:
        for potential_album in albums:
            if artist in album.artist_set.all():
                album = potential_album
    if not album:
        album_created = True
        album = Album.objects.create(name=name, musicbrainz_id=mbid)
        album.save()
        album.artists.add(artist)

    if album_created or not mbid:
        album_dict = lookup_album_dict_from_mb(
            album.name, artist_name=artist.name
        )
        album.year = album_dict["year"]
        album.musicbrainz_id = album_dict["mb_id"]
        album.musicbrainz_releasegroup_id = album_dict["mb_group_id"]
        album.musicbrainz_albumartist_id = artist.musicbrainz_id
        album.save(
            update_fields=[
                "year",
                "musicbrainz_id",
                "musicbrainz_releasegroup_id",
                "musicbrainz_albumartist_id",
            ]
        )
        album.artists.add(artist)
        album.fetch_artwork()
    return album


def get_or_create_track(
    title: str,
    artist: Artist,
    album: Album,
    mbid: str = None,
    run_time=None,
    run_time_ticks=None,
) -> Track:
    track = None
    if not mbid:
        mbid = lookup_track_from_mb(
            title,
            artist.musicbrainz_id,
            album.musicbrainz_id,
        )['id']

    track = Track.objects.filter(musicbrainz_id=mbid).first()

    if not track:
        track = Track.objects.create(
            title=title,
            artist=artist,
            album=album,
            musicbrainz_id=mbid,
            run_time=run_time,
            run_time_ticks=run_time_ticks,
        )

    return track



from music.models import Album, Artist, Track
from rest_framework import serializers


class ArtistSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Artist
        fields = "__all__"


class AlbumSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Album
        fields = "__all__"


class TrackSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Track
        fields = "__all__"



# cli.py
import logging
import sys
from os import environ as env


if not 'DJANGO_SETTINGS_MODULE' in env:
    from vrobbler import settings

    env.setdefault('DJANGO_SETTINGS_MODULE', settings.__name__)


import django

django.setup()

# this line must be after django.setup() for logging configure
logger = logging.getLogger('vrobbler')


def main():
    # to get configured settings
    from django.conf import settings

    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()



import logging
from tempfile import NamedTemporaryFile
from typing import Dict, Optional
from urllib.request import urlopen
from uuid import uuid4

import musicbrainzngs
from django.conf import settings
from django.core.files.base import File
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from scrobbles.mixins import ScrobblableMixin
from scrobbles.theaudiodb import lookup_artist_from_tadb

logger = logging.getLogger(__name__)
BNULL = {"blank": True, "null": True}


class Artist(TimeStampedModel):
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)
    name = models.CharField(max_length=255)
    biography = models.TextField(**BNULL)
    theaudiodb_genre = models.CharField(max_length=255, **BNULL)
    theaudiodb_mood = models.CharField(max_length=255, **BNULL)
    musicbrainz_id = models.CharField(max_length=255, **BNULL)
    thumbnail = models.ImageField(upload_to="artist/", **BNULL)

    class Meta:
        unique_together = [['name', 'musicbrainz_id']]

    def __str__(self):
        return self.name

    @property
    def mb_link(self):
        return f"https://musicbrainz.org/artist/{self.musicbrainz_id}"

    def get_absolute_url(self):
        return reverse('music:artist_detail', kwargs={'slug': self.uuid})

    def scrobbles(self):
        from scrobbles.models import Scrobble

        return Scrobble.objects.filter(
            track__in=self.track_set.all()
        ).order_by('-timestamp')

    @property
    def tracks(self):
        return (
            self.track_set.all()
            .annotate(scrobble_count=models.Count('scrobble'))
            .order_by('-scrobble_count')
        )

    def charts(self):
        from scrobbles.models import ChartRecord

        return ChartRecord.objects.filter(track__artist=self).order_by('-year')

    def fix_metadata(self):
        tadb_info = lookup_artist_from_tadb(self.name)
        if not tadb_info:
            logger.warn(f"No response from TADB for artist {self.name}")
            return

        self.biography = tadb_info['biography']
        self.theaudiodb_genre = tadb_info['genre']
        self.theaudiodb_mood = tadb_info['mood']

        img_temp = NamedTemporaryFile(delete=True)
        img_temp.write(urlopen(tadb_info['thumb_url']).read())
        img_temp.flush()
        img_filename = f"{self.name}_{self.uuid}.jpg"
        self.thumbnail.save(img_filename, File(img_temp))


class Album(TimeStampedModel):
    uuid = models.UUIDField(default=uuid4, editable=False, **BNULL)
    name = models.CharField(max_length=255)
    artists = models.ManyToManyField(Artist)
    year = models.IntegerField(**BNULL)
    musicbrainz_id = models.CharField(max_length=255, unique=True, **BNULL)
    musicbrainz_releasegroup_id = models.CharField(max_length=255, **BNULL)
    musicbrainz_albumartist_id = models.CharField(max_length=255, **BNULL)
    cover_image = models.ImageField(upload_to="albums/", **BNULL)

    def __str__(self):
        return self.name

    @property
    def primary_artist(self):
        return self.artists.first()

    def fix_metadata(self):
        if (
            not self.musicbrainz_albumartist_id
            or not self.year
            or not self.musicbrainz_releasegroup_id
        ):
            musicbrainzngs.set_useragent('vrobbler', '0.3.0')
            mb_data = musicbrainzngs.get_release_by_id(
                self.musicbrainz_id, includes=['artists', 'release-groups']
            )
            if not self.musicbrainz_releasegroup_id:
                self.musicbrainz_releasegroup_id = mb_data['release'][
                    'release-group'
                ]['id']
            if not self.musicbrainz_albumartist_id:
                self.musicbrainz_albumartist_id = mb_data['release'][
                    'artist-credit'
                ][0]['artist']['id']
            if not self.year:
                try:
                    self.year = mb_data['release']['date'][0:4]
                except KeyError:
                    pass
                except IndexError:
                    pass

            self.save(
                update_fields=[
                    'musicbrainz_albumartist_id',
                    'musicbrainz_releasegroup_id',
                    'year',
                ]
            )

            new_artist = Artist.objects.filter(
                musicbrainz_id=self.musicbrainz_albumartist_id
            ).first()
            if self.musicbrainz_albumartist_id and new_artist:
                self.artists.add(new_artist)
            if not new_artist:
                for t in self.track_set.all():
                    self.artists.add(t.artist)
            if (
                not self.cover_image
                or self.cover_image == 'default-image-replace-me'
            ):
                self.fetch_artwork()

    def fetch_artwork(self, force=False):
        if not self.cover_image and not force:
            if self.musicbrainz_id:
                try:
                    img_data = musicbrainzngs.get_image_front(
                        self.musicbrainz_id
                    )
                    name = f"{self.name}_{self.uuid}.jpg"
                    self.cover_image = ContentFile(img_data, name=name)
                    logger.info(f'Setting image to {name}')
                except musicbrainzngs.ResponseError:
                    logger.warning(
                        f'No cover art found for {self.name} by release'
                    )

            if (
                not self.cover_image
                or self.cover_image == "default-image-replace-me"
            ) and self.musicbrainz_releasegroup_id:
                try:
                    img_data = musicbrainzngs.get_release_group_image_front(
                        self.musicbrainz_releasegroup_id
                    )
                    name = f"{self.name}_{self.uuid}.jpg"
                    self.cover_image = ContentFile(img_data, name=name)
                    logger.info(f'Setting image to {name}')
                except musicbrainzngs.ResponseError:
                    logger.warning(
                        f'No cover art found for {self.name} by release group'
                    )
            if not self.cover_image:
                logger.debug(
                    f"No cover art found for release or release group for {self.name}, setting to default"
                )
            self.save()

    @property
    def mb_link(self):
        return f"https://musicbrainz.org/release/{self.musicbrainz_id}"


class Track(ScrobblableMixin):
    COMPLETION_PERCENT = getattr(settings, 'MUSIC_COMPLETION_PERCENT', 90)

    class Opinion(models.IntegerChoices):
        DOWN = -1, 'Thumbs down'
        NEUTRAL = 0, 'No opinion'
        UP = 1, 'Thumbs up'

    artist = models.ForeignKey(Artist, on_delete=models.DO_NOTHING)
    album = models.ForeignKey(Album, on_delete=models.DO_NOTHING, **BNULL)
    musicbrainz_id = models.CharField(max_length=255, **BNULL)

    class Meta:
        unique_together = [['album', 'musicbrainz_id']]

    def __str__(self):
        return f"{self.title} by {self.artist}"

    def get_absolute_url(self):
        return reverse('music:track_detail', kwargs={'slug': self.uuid})

    @property
    def subtitle(self):
        return self.artist

    @property
    def mb_link(self):
        return f"https://musicbrainz.org/recording/{self.musicbrainz_id}"

    @property
    def info_link(self):
        return self.mb_link

    @classmethod
    def find_or_create(
        cls, artist_dict: Dict, album_dict: Dict, track_dict: Dict
    ) -> Optional["Track"]:
        """Given a data dict from Jellyfin, does the heavy lifting of looking up
        the video and, if need, TV Series, creating both if they don't yet
        exist.

        """
        if not artist_dict.get('name') or not artist_dict.get(
            'musicbrainz_id'
        ):
            logger.warning(
                f"No artist or artist musicbrainz ID found in message from source, not scrobbling"
            )
            return

        artist, artist_created = Artist.objects.get_or_create(**artist_dict)
        album, album_created = Album.objects.get_or_create(**album_dict)

        album.fix_metadata()
        if not album.cover_image:
            album.fetch_artwork()

        track_dict['album_id'] = getattr(album, "id", None)
        track_dict['artist_id'] = artist.id

        track, created = cls.objects.get_or_create(**track_dict)

        return track



import csv
import logging
from datetime import datetime

import pytz
from scrobbles.models import Scrobble
from music.utils import (
    get_or_create_album,
    get_or_create_artist,
    get_or_create_track,
)

logger = logging.getLogger(__name__)


def process_audioscrobbler_tsv_file(file_path, user_id, user_tz=None):
    """Takes a path to a file of TSV data and imports it as past scrobbles"""
    new_scrobbles = []
    if not user_tz:
        user_tz = pytz.utc

    with open(file_path) as infile:
        source = 'Audioscrobbler File'
        rows = csv.reader(infile, delimiter="\t")

        source_id = ""
        for row_num, row in enumerate(rows):
            if row_num in [0, 1, 2]:
                if "Rockbox" in row[0]:
                    source = "Rockbox"
                source_id += row[0] + "\n"
                continue
            if len(row) > 8:
                logger.warning(
                    'Improper row length during Audioscrobbler import',
                    extra={'row': row},
                )
                continue
            artist = get_or_create_artist(row[0])
            album = get_or_create_album(row[1], artist)

            track = get_or_create_track(
                title=row[2],
                mbid=row[7],
                artist=artist,
                album=album,
                run_time=row[4],
                run_time_ticks=int(row[4]) * 1000,
            )

            timestamp = (
                datetime.utcfromtimestamp(int(row[6]))
                .replace(tzinfo=user_tz)
                .astimezone(pytz.utc)
            )

            new_scrobble = Scrobble(
                user_id=user_id,
                timestamp=timestamp,
                source=source,
                source_id=source_id,
                track=track,
                played_to_completion=True,
                in_progress=False,
            )
            existing = Scrobble.objects.filter(
                timestamp=timestamp, track=track
            ).first()
            if existing:
                logger.debug(f"Skipping existing scrobble {new_scrobble}")
                continue
            logger.debug(f"Queued scrobble {new_scrobble} for creation")
            new_scrobbles.append(new_scrobble)

        created = Scrobble.objects.bulk_create(new_scrobbles)
        logger.info(
            f"Created {len(created)} scrobbles",
            extra={'created_scrobbles': created},
        )
        return created



from rest_framework.negotiation import BaseContentNegotiation


class IgnoreClientContentNegotiation(BaseContentNegotiation):
    def select_parser(self, request, parsers):
        """
        Select the first parser in the `.parser_classes` list.
        """
        return parsers[0]

    def select_renderer(self, request, renderers, format_suffix):
        """
        Select the first renderer in the `.renderer_classes` list.
        """
        return (renderers[0], renderers[0].media_type)



from rest_framework import permissions, viewsets

from music.api.serializers import (
    TrackSerializer,
    ArtistSerializer,
    AlbumSerializer,
)
from music.models import Artist, Album, Track


class ArtistViewSet(viewsets.ModelViewSet):
    queryset = Artist.objects.all().order_by('-created')
    serializer_class = ArtistSerializer
    permission_classes = [permissions.IsAuthenticated]


class AlbumViewSet(viewsets.ModelViewSet):
    queryset = Album.objects.all().order_by('-created')
    serializer_class = AlbumSerializer
    permission_classes = [permissions.IsAuthenticated]


class TrackViewSet(viewsets.ModelViewSet):
    queryset = Track.objects.all().order_by('-created')
    serializer_class = TrackSerializer
    permission_classes = [permissions.IsAuthenticated]



# Generated by Django 4.1.5 on 2023-02-23 15:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0007_sport_default_event_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='sportevent',
            name='thesportsdb_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



import calendar
import json
import logging
from datetime import datetime, timedelta
from django.db.models.query import QuerySet

import pytz
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.db.models import Q
from django.db.models.fields import timezone
from django.http import FileResponse, HttpResponseRedirect, JsonResponse
from django.urls import reverse, reverse_lazy
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import DetailView, FormView, TemplateView
from django.views.generic.edit import CreateView
from django.views.generic.list import ListView
from music.aggregators import (
    scrobble_counts,
    top_artists,
    top_tracks,
    week_of_scrobbles,
)
from rest_framework import status
from rest_framework.decorators import (
    api_view,
    parser_classes,
    permission_classes,
)
from rest_framework.parsers import MultiPartParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from scrobbles.api import serializers
from scrobbles.constants import (
    JELLYFIN_AUDIO_ITEM_TYPES,
    JELLYFIN_VIDEO_ITEM_TYPES,
)
from scrobbles.export import export_scrobbles
from scrobbles.forms import ExportScrobbleForm, ScrobbleForm
from scrobbles.imdb import lookup_video_from_imdb
from scrobbles.models import (
    AudioScrobblerTSVImport,
    ChartRecord,
    KoReaderImport,
    LastFmImport,
    Scrobble,
)
from scrobbles.scrobblers import (
    jellyfin_scrobble_track,
    jellyfin_scrobble_video,
    manual_scrobble_event,
    manual_scrobble_video,
    mopidy_scrobble_podcast,
    mopidy_scrobble_track,
)
from scrobbles.tasks import (
    process_koreader_import,
    process_lastfm_import,
    process_tsv_import,
)
from scrobbles.thesportsdb import lookup_event_from_thesportsdb

logger = logging.getLogger(__name__)


class RecentScrobbleList(ListView):
    model = Scrobble

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        user = self.request.user
        if user.is_authenticated:

            completed_for_user = Scrobble.objects.filter(
                played_to_completion=True, user=user
            )
            data['video_scrobble_list'] = completed_for_user.filter(
                video__isnull=False
            ).order_by('-timestamp')[:15]

            data['podcast_scrobble_list'] = completed_for_user.filter(
                podcast_episode__isnull=False
            ).order_by('-timestamp')[:15]

            data['sport_scrobble_list'] = completed_for_user.filter(
                sport_event__isnull=False
            ).order_by('-timestamp')[:15]

        data["weekly_data"] = week_of_scrobbles(user=user)

        data['counts'] = scrobble_counts(user)
        data['imdb_form'] = ScrobbleForm
        data['export_form'] = ExportScrobbleForm
        data['active_imports'] = AudioScrobblerTSVImport.objects.filter(
            processing_started__isnull=False,
            processed_finished__isnull=True,
            user=self.request.user,
        )
        return data

    def get_queryset(self):
        return Scrobble.objects.filter(
            track__isnull=False, in_progress=False
        ).order_by('-timestamp')[:15]


class ScrobbleImportListView(TemplateView):
    template_name = "scrobbles/import_list.html"

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)
        context_data['object_list'] = []

        context_data["tsv_imports"] = AudioScrobblerTSVImport.objects.filter(
            user=self.request.user,
        ).order_by('-processing_started')
        context_data["koreader_imports"] = KoReaderImport.objects.filter(
            user=self.request.user,
        ).order_by('-processing_started')
        context_data["lastfm_imports"] = LastFmImport.objects.filter(
            user=self.request.user,
        ).order_by('-processing_started')
        return context_data


class BaseScrobbleImportDetailView(DetailView):
    slug_field = 'uuid'
    template_name = "scrobbles/import_detail.html"

    def get_queryset(self):
        return super().get_queryset().filter(user=self.request.user)

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)
        title = "Generic Scrobble Import"
        if self.model == KoReaderImport:
            title = "KoReader Import"
        if self.model == AudioScrobblerTSVImport:
            title = "Audioscrobbler TSV Import"
        if self.model == LastFmImport:
            title = "LastFM Import"
        context_data['title'] = title
        return context_data


class ScrobbleKoReaderImportDetailView(BaseScrobbleImportDetailView):
    model = KoReaderImport


class ScrobbleTSVImportDetailView(BaseScrobbleImportDetailView):
    model = AudioScrobblerTSVImport


class ScrobbleLastFMImportDetailView(BaseScrobbleImportDetailView):
    model = LastFmImport


class ManualScrobbleView(FormView):
    form_class = ScrobbleForm
    template_name = 'scrobbles/manual_form.html'

    def form_valid(self, form):

        item_id = form.cleaned_data.get('item_id')
        data_dict = None
        if 'tt' in item_id:
            data_dict = lookup_video_from_imdb(
                form.cleaned_data.get('item_id')
            )
            if data_dict:
                manual_scrobble_video(data_dict, self.request.user.id)

        if not data_dict:
            logger.debug(f"Looking for sport event with ID {item_id}")
            data_dict = lookup_event_from_thesportsdb(
                form.cleaned_data.get('item_id')
            )
            if data_dict:
                manual_scrobble_event(data_dict, self.request.user.id)

        return HttpResponseRedirect(reverse("vrobbler-home"))


class JsonableResponseMixin:
    """
    Mixin to add JSON support to a form.
    Must be used with an object-based FormView (e.g. CreateView)
    """

    def form_invalid(self, form):
        response = super().form_invalid(form)
        if self.request.accepts('text/html'):
            return response
        else:
            return JsonResponse(form.errors, status=400)

    def form_valid(self, form):
        # We make sure to call the parent's form_valid() method because
        # it might do some processing (in the case of CreateView, it will
        # call form.save() for example).
        response = super().form_valid(form)
        if self.request.accepts('text/html'):
            return response
        else:
            data = {
                'pk': self.object.pk,
            }
            return JsonResponse(data)


class AudioScrobblerImportCreateView(
    LoginRequiredMixin, JsonableResponseMixin, CreateView
):
    model = AudioScrobblerTSVImport
    fields = ['tsv_file']
    template_name = 'scrobbles/upload_form.html'
    success_url = reverse_lazy('vrobbler-home')

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        process_tsv_import.delay(self.object.id)
        return HttpResponseRedirect(self.get_success_url())


class KoReaderImportCreateView(
    LoginRequiredMixin, JsonableResponseMixin, CreateView
):
    model = KoReaderImport
    fields = ['sqlite_file']
    template_name = 'scrobbles/upload_form.html'
    success_url = reverse_lazy('vrobbler-home')

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        process_koreader_import.delay(self.object.id)
        return HttpResponseRedirect(self.get_success_url())


@permission_classes([IsAuthenticated])
@api_view(['GET'])
def lastfm_import(request):
    lfm_import, created = LastFmImport.objects.get_or_create(
        user=request.user, processed_finished__isnull=True
    )

    process_lastfm_import.delay(lfm_import.id)

    success_url = reverse_lazy('vrobbler-home')
    return HttpResponseRedirect(success_url)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
def jellyfin_webhook(request):
    data_dict = request.data

    if (
        data_dict['NotificationType'] == 'PlaybackProgress'
        and data_dict['ItemType'] == 'Audio'
    ):
        return Response({}, status=status.HTTP_304_NOT_MODIFIED)

    # For making things easier to build new input processors
    if getattr(settings, "DUMP_REQUEST_DATA", False):
        json_data = json.dumps(data_dict, indent=4)
        logger.debug(f"{json_data}")

    scrobble = None
    media_type = data_dict.get("ItemType", "")

    if media_type in JELLYFIN_AUDIO_ITEM_TYPES:
        scrobble = jellyfin_scrobble_track(data_dict, request.user.id)

    if media_type in JELLYFIN_VIDEO_ITEM_TYPES:
        scrobble = jellyfin_scrobble_video(data_dict, request.user.id)

    if not scrobble:
        return Response({}, status=status.HTTP_400_BAD_REQUEST)

    return Response({'scrobble_id': scrobble.id}, status=status.HTTP_200_OK)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
def mopidy_webhook(request):
    try:
        data_dict = json.loads(request.data)
    except TypeError:
        logger.warning('Received Mopidy data as dict, rather than a string')
        data_dict = request.data

    # For making things easier to build new input processors
    if getattr(settings, "DUMP_REQUEST_DATA", False):
        json_data = json.dumps(data_dict, indent=4)
        logger.debug(f"{json_data}")

    if 'podcast' in data_dict.get('mopidy_uri'):
        scrobble = mopidy_scrobble_podcast(data_dict, request.user.id)
    else:
        scrobble = mopidy_scrobble_track(data_dict, request.user.id)

    if not scrobble:
        return Response({}, status=status.HTTP_400_BAD_REQUEST)

    return Response({'scrobble_id': scrobble.id}, status=status.HTTP_200_OK)


@csrf_exempt
@permission_classes([IsAuthenticated])
@api_view(['POST'])
@parser_classes([MultiPartParser])
def import_audioscrobbler_file(request):
    """Takes a TSV file in the Audioscrobbler format, saves it and processes the
    scrobbles.
    """
    scrobbles_created = []
    # tsv_file = request.FILES[0]

    file_serializer = serializers.AudioScrobblerTSVImportSerializer(
        data=request.data
    )
    if file_serializer.is_valid():
        import_file = file_serializer.save()
        return Response(
            {'scrobble_ids': scrobbles_created}, status=status.HTTP_200_OK
        )
    else:
        return Response(
            file_serializer.errors, status=status.HTTP_400_BAD_REQUEST
        )


@permission_classes([IsAuthenticated])
@api_view(['GET'])
def scrobble_finish(request, uuid):
    user = request.user
    success_url = reverse_lazy('vrobbler-home')

    if not user.is_authenticated:
        return HttpResponseRedirect(success_url)

    scrobble = Scrobble.objects.filter(user=user, uuid=uuid).first()
    if scrobble:
        scrobble.stop(force_finish=True)
        messages.add_message(
            request,
            messages.SUCCESS,
            f"Scrobble of {scrobble.media_obj} finished.",
        )
    else:
        messages.add_message(request, messages.ERROR, "Scrobble not found.")
    return HttpResponseRedirect(success_url)


@permission_classes([IsAuthenticated])
@api_view(['GET'])
def scrobble_cancel(request, uuid):
    user = request.user
    success_url = reverse_lazy('vrobbler-home')

    if not user.is_authenticated:
        return HttpResponseRedirect(success_url)

    scrobble = Scrobble.objects.filter(user=user, uuid=uuid).first()
    if scrobble:
        scrobble.cancel()
        messages.add_message(
            request,
            messages.SUCCESS,
            f"Scrobble of {scrobble.media_obj} cancelled.",
        )
    else:
        messages.add_message(request, messages.ERROR, "Scrobble not found.")
    return HttpResponseRedirect(success_url)


@permission_classes([IsAuthenticated])
@api_view(['GET'])
def export(request):
    format = request.GET.get('export_type', 'csv')
    start = request.GET.get('start')
    end = request.GET.get('end')
    logger.debug(f"Exporting all scrobbles in format {format}")

    temp_file, extension = export_scrobbles(
        start_date=start, end_date=end, format=format
    )

    now = datetime.now()
    filename = f"vrobbler-export-{str(now)}.{extension}"
    response = FileResponse(open(temp_file, 'rb'))
    response["Content-Disposition"] = f'attachment; filename="{filename}"'

    return response


class ChartRecordView(TemplateView):
    template_name = 'scrobbles/chart_index.html'

    @staticmethod
    def get_media_filter(media_type: str = "Track"):
        media_filter = Q()
        if media_type == 'Track':
            media_filter = Q(track__isnull=False)
        if media_type == 'Artist':
            media_filter = Q(artist__isnull=False)
        if media_type == 'Series':
            media_filter = Q(series__isnull=False)
        if media_type == 'Video':
            media_filter = Q(video__isnull=False)
        return media_filter

    def get_chart_records(self, media_type: str = "Track", **kwargs):
        media_filter = self.get_media_filter(media_type)
        charts = ChartRecord.objects.filter(
            media_filter, user=self.request.user, **kwargs
        ).order_by("rank")

        if charts.count() == 0:
            ChartRecord.build(
                user=self.request.user, model_str=media_type, **kwargs
            )
            charts = ChartRecord.objects.filter(
                media_filter, user=self.request.user, **kwargs
            ).order_by("rank")
        return charts

    def get_chart(
        self, period: str = "all_time", limit=15, media: str = "Track"
    ) -> QuerySet:
        chart = QuerySet()
        now = timezone.now()
        if period == "all_time":
            chart = self.get_chart_records(media_type=media)
        if period == "today":
            chart = self.get_chart_records(
                media_type=media,
                day=now.day,
                month=now.month,
                year=now.year,
            )
        if period == "week":
            chart = self.get_chart_records(
                media_type=media,
                year=now.year,
                week=now.isocalendar()[1],
            )
        if period == "month":
            chart = self.get_chart_records(
                media_type=media,
                year=now.year,
                month=now.month,
            )
        if period == "year":
            chart = self.get_chart_records(
                media_type=media,
                year=now.year,
            )
        return chart[:limit]

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)
        date = self.request.GET.get('date')
        media_type = self.request.GET.get('media')
        user = self.request.user
        params = {}
        context_data['artist_charts'] = {}

        if not date:
            context_data['artist_charts'] = {
                "today": top_artists(user, filter="today")[:30],
                "week": top_artists(user, filter="week")[:30],
                "month": top_artists(user, filter="month")[:30],
                "all": top_artists(user),
            }

            context_data['track_charts'] = {
                "today": top_tracks(user, filter="today")[:30],
                "week": top_tracks(user, filter="week")[:30],
                "month": top_tracks(user, filter="month")[:30],
                "all": top_tracks(user),
            }
            return context_data

        now = timezone.now()
        year = now.year
        params = {'year': year}
        name = f"Chart for {year}"

        date_params = date.split('-')
        year = int(date_params[0])
        in_progress = False
        if len(date_params) == 2:
            if 'W' in date_params[1]:
                week = int(date_params[1].strip('W"'))
                params['week'] = week
                start = datetime.strptime(date + "-1", "%Y-W%W-%w").replace(
                    tzinfo=pytz.utc
                )
                end = start + timedelta(days=6)
                in_progress = start <= now <= end
                as_str = start.strftime('Week of %B %d, %Y')
                name = f"Chart for {as_str}"
            else:
                month = int(date_params[1])
                params['month'] = month
                month_str = calendar.month_name[month]
                name = f"Chart for {month_str} {year}"
                in_progress = now.month == month and now.year == year
        if len(date_params) == 3:
            month = int(date_params[1])
            day = int(date_params[2])
            params['month'] = month
            params['day'] = day
            month_str = calendar.month_name[month]
            name = f"Chart for {month_str} {day}, {year}"
            in_progress = (
                now.month == month and now.year == year and now.day == day
            )

        media_filter = self.get_media_filter(media_type)
        charts = ChartRecord.objects.filter(
            media_filter, user=self.request.user, **params
        ).order_by("rank")

        if charts.count() == 0:
            ChartRecord.build(
                user=self.request.user, model_str=media_type, **params
            )
            charts = ChartRecord.objects.filter(
                media_filter, user=self.request.user, **params
            ).order_by("rank")

        if in_progress:
            # TODO recalculate
            ...

        context_data['name'] = name
        context_data['in_progress'] = in_progress
        return context_data



# Generated by Django 4.1.5 on 2023-01-08 01:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='album',
            name='year',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



import pytz
from django.utils import timezone
from scrobbles.models import Scrobble


def now_playing(request):
    user = request.user
    now = timezone.now()
    if user.is_authenticated:
        if user.profile:
            timezone.activate(pytz.timezone(user.profile.timezone))
            now = timezone.localtime(timezone.now())
        return {
            'now_playing_list': Scrobble.objects.filter(
                in_progress=True,
                is_paused=False,
                user=user,
            )
        }



from django.contrib import admin
from podcasts.models import Episode, Podcast, Producer
from scrobbles.admin import ScrobbleInline


@admin.register(Producer)
class ProducerAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = ("name",)
    ordering = ("name",)


@admin.register(Podcast)
class PodcastAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = (
        "name",
        "producer",
        "active",
    )
    ordering = ("name",)


@admin.register(Episode)
class EpisodeAdmin(admin.ModelAdmin):
    date_hierarchy = "created"
    list_display = (
        "title",
        "podcast",
        "run_time",
    )
    list_filter = ("podcast",)
    ordering = ("-created",)
    inlines = [
        ScrobbleInline,
    ]



# Generated by Django 4.1.5 on 2023-02-03 23:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0015_audioscrobblertsvimport_uuid_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='audioscrobblertsvimport',
            name='process_count',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



import json
import logging

import requests
from django.conf import settings

THEAUDIODB_API_KEY = getattr(settings, "THEAUDIODB_API_KEY")
SEARCH_URL = f"https://www.theaudiodb.com/api/v1/json/{THEAUDIODB_API_KEY}/search.php?s="

logger = logging.getLogger(__name__)


def lookup_artist_from_tadb(name: str) -> dict:
    artist_info = {}
    response = requests.get(SEARCH_URL + name)

    if response.status_code != 200:
        logger.warn(f"Bad response from TADB: {response.status_code}")
        return {}

    if not response.content:
        logger.warn(f"Bad content from TADB: {response.content}")
        return {}

    results = json.loads(response.content)
    artist = results['artists'][0]

    artist_info['biography'] = artist['strBiographyEN']
    artist_info['genre'] = artist['strGenre']
    artist_info['mood'] = artist['strMood']
    artist_info['thumb_url'] = artist['strArtistThumb']

    return artist_info



# Generated by Django 4.1.5 on 2023-02-16 17:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0018_lastfmimport'),
    ]

    operations = [
        migrations.RenameField(
            model_name='lastfmimport',
            old_name='processed_on',
            new_name='processed_finished',
        ),
        migrations.AddField(
            model_name='lastfmimport',
            name='processing_started',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from django.contrib.auth import get_user_model

from rest_framework import permissions, viewsets

from profiles.api.serializers import UserSerializer, UserProfileSerializer
from profiles.models import UserProfile

User = get_user_model()


class UserViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """

    queryset = User.objects.all().order_by('-date_joined')
    serializer_class = UserSerializer
    permission_classes = [permissions.IsAuthenticated]


class UserProfileViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """

    queryset = UserProfile.objects.all().order_by('-created')
    serializer_class = UserProfileSerializer
    permission_classes = [permissions.IsAuthenticated]



# Generated by Django 4.1.5 on 2023-01-30 17:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('scrobbles', '0010_chartrecord'),
    ]

    operations = [
        migrations.AddField(
            model_name='chartrecord',
            name='user',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from django.db.models import Count
from django.views import generic
from music.models import Track, Artist, Album
from scrobbles.stats import get_scrobble_count_qs


class TrackListView(generic.ListView):
    model = Track

    def get_queryset(self):
        return get_scrobble_count_qs(user=self.request.user).order_by(
            "-scrobble_count"
        )


class TrackDetailView(generic.DetailView):
    model = Track
    slug_field = 'uuid'


class ArtistListView(generic.ListView):
    model = Artist

    def get_queryset(self):
        return super().get_queryset().order_by("name")


class ArtistDetailView(generic.DetailView):
    model = Artist
    slug_field = 'uuid'


class AlbumListView(generic.ListView):
    model = Album



# Generated by Django 4.1.5 on 2023-02-19 03:53

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import scrobbles.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        (
            'scrobbles',
            '0019_rename_processed_on_lastfmimport_processed_finished_and_more',
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='audioscrobblertsvimport',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lastfmimport',
            options={},
        ),
        migrations.RenameField(
            model_name='audioscrobblertsvimport',
            old_name='processed_on',
            new_name='processed_finished',
        ),
        migrations.AddField(
            model_name='audioscrobblertsvimport',
            name='processing_started',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name='KoReaderImport',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'created',
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name='created'
                    ),
                ),
                (
                    'modified',
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name='modified'
                    ),
                ),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                (
                    'processing_started',
                    models.DateTimeField(blank=True, null=True),
                ),
                (
                    'processed_finished',
                    models.DateTimeField(blank=True, null=True),
                ),
                ('process_log', models.TextField(blank=True, null=True)),
                ('process_count', models.IntegerField(blank=True, null=True)),
                (
                    'sqlite_file',
                    models.FileField(
                        blank=True,
                        null=True,
                        upload_to=scrobbles.models.KoReaderImport.get_path,
                    ),
                ),
                (
                    'user',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging

import musicbrainzngs
from dateutil.parser import parse

logger = logging.getLogger(__name__)


def lookup_album_from_mb(musicbrainz_id: str) -> dict:
    release_dict = {}

    musicbrainzngs.set_useragent('vrobbler', '0.3.0')
    release_data = musicbrainzngs.get_release_by_id(
        musicbrainz_id,
        includes=['artists', 'release-groups', 'recordings'],
    ).get('release')

    if not release_data:
        return release_dict

    primary_artist = release_data.get('artist-credit')[0]
    release_dict = {
        'artist': {
            'name': primary_artist.get('name'),
            'musicbrainz_id': primary_artist.get('id'),
        },
        'album': {
            'name': release_data.get('title'),
            'musicbrainz_id': musicbrainz_id,
            'musicbrainz_releasegroup_id': release_data.get(
                'release-group'
            ).get('id'),
            'musicbrainz_albumaritist_id': primary_artist.get('id'),
            'year': release_data.get('year')[0:4],
        },
    }

    release_dict['tracks'] = []
    for track in release_data.get('medium-list')[0]['track-list']:
        recording = track['recording']
        release_dict['tracks'].append(
            {
                'title': recording['title'],
                'musicbrainz_id': recording['id'],
                'run_time_ticks': track['length'],
            }
        )

    return release_dict


def lookup_album_dict_from_mb(release_name: str, artist_name: str) -> dict:
    musicbrainzngs.set_useragent('vrobbler', '0.3.0')

    top_result = musicbrainzngs.search_releases(
        release_name, artist=artist_name
    )['release-list'][0]
    score = int(top_result.get('ext:score'))
    if score < 85:
        logger.debug(
            "Album lookup score below 85 threshold",
            extra={"result": top_result},
        )
        return {}
    year = None
    if top_result.get("date"):
        year = parse(top_result["date"]).year

    return {
        "year": year,
        "mb_id": top_result["id"],
        "mb_group_id": top_result["release-group"]["id"],
    }


def lookup_artist_from_mb(artist_name: str) -> str:
    musicbrainzngs.set_useragent('vrobbler', '0.3.0')

    top_result = musicbrainzngs.search_artists(artist=artist_name)[
        'artist-list'
    ][0]
    score = int(top_result.get('ext:score'))
    if score < 85:
        logger.debug(
            "Artist lookup score below 85 threshold",
            extra={"result": top_result},
        )
        return ""

    return top_result


def lookup_track_from_mb(
    track_name: str, artist_mbid: str, album_mbid: str
) -> str:
    musicbrainzngs.set_useragent('vrobbler', '0.3.0')

    top_result = musicbrainzngs.search_recordings(
        query=track_name, artist=artist_mbid, release=album_mbid
    )['recording-list'][0]
    score = int(top_result.get('ext:score'))
    if score < 85:
        logger.debug(
            "Track lookup score below 85 threshold",
            extra={"result": top_result},
        )
        return ""

    return top_result



# Generated by Django 4.1.5 on 2023-02-12 22:26

from django.db import migrations, models
import encrypted_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='lastfm_password',
            field=encrypted_field.fields.EncryptedField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='lastfm_username',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



import json
from typing import Optional
import requests
import logging

logger = logging.getLogger(__name__)

SEARCH_URL = "https://openlibrary.org/search.json?title={title}"
ISBN_URL = "https://openlibrary.org/isbn/{isbn}.json"


def get_first(key: str, result: dict) -> str:
    obj = ""
    if obj_list := result.get(key):
        obj = obj_list[0]
    return obj


def lookup_book_from_openlibrary(title: str, author: str = None) -> dict:
    search_url = SEARCH_URL.format(title=title)
    response = requests.get(search_url)

    if response.status_code != 200:
        logger.warn(f"Bad response from OL: {response.status_code}")
        return {}

    results = json.loads(response.content)

    if len(results.get('docs')) == 0:
        logger.warn(f"No results found from OL for {title}")
        return {}

    top = results.get('docs')[0]
    if author and author not in top['author_name']:
        logger.warn(
            f"Lookup for {title} found top result with mismatched author"
        )

    return {
        "title": top.get("title"),
        "isbn": top.get("isbn")[0],
        "openlibrary_id": top.get("cover_edition_key"),
        "author_name": get_first("author_name", top),
        "author_openlibrary_id": get_first("author_key", top),
        "goodreads_id": get_first("id_goodreads", top),
        "first_publish_year": top.get("first_publish_year"),
    }



# Generated by Django 4.1.5 on 2023-02-25 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0022_scrobble_book_pages_read'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='audioscrobblertsvimport',
            options={'verbose_name': 'AudioScrobbler TSV Import'},
        ),
        migrations.AlterModelOptions(
            name='koreaderimport',
            options={'verbose_name': 'KOReader Import'},
        ),
        migrations.AlterModelOptions(
            name='lastfmimport',
            options={'verbose_name': 'Last.FM Import'},
        ),
        migrations.AddField(
            model_name='chartrecord',
            name='count',
            field=models.IntegerField(default=0),
        ),
    ]



from datetime import datetime, timedelta
from typing import List, Optional

import pytz
from django.db.models import Count, Q, Sum
from django.utils import timezone
from music.models import Artist, Track
from scrobbles.models import Scrobble
from videos.models import Video
from vrobbler.apps.profiles.utils import now_user_timezone

NOW = timezone.now()
START_OF_TODAY = datetime.combine(NOW.date(), datetime.min.time(), NOW.tzinfo)
STARTING_DAY_OF_CURRENT_WEEK = NOW.date() - timedelta(
    days=NOW.today().isoweekday() % 7
)
STARTING_DAY_OF_CURRENT_MONTH = NOW.date().replace(day=1)
STARTING_DAY_OF_CURRENT_YEAR = NOW.date().replace(month=1, day=1)


def scrobble_counts(user=None):

    now = timezone.now()
    user_filter = Q()
    if user and user.is_authenticated:
        now = now_user_timezone(user.profile)
        user_filter = Q(user=user)

    start_of_today = datetime.combine(
        now.date(), datetime.min.time(), now.tzinfo
    )
    starting_day_of_current_week = now.date() - timedelta(
        days=now.today().isoweekday() % 7
    )
    starting_day_of_current_month = now.date().replace(day=1)
    starting_day_of_current_year = now.date().replace(month=1, day=1)

    finished_scrobbles_qs = Scrobble.objects.filter(
        user_filter, played_to_completion=True
    )
    data = {}
    data['today'] = finished_scrobbles_qs.filter(
        timestamp__gte=start_of_today
    ).count()
    data['week'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_week
    ).count()
    data['month'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_month
    ).count()
    data['year'] = finished_scrobbles_qs.filter(
        timestamp__gte=starting_day_of_current_year
    ).count()
    data['alltime'] = finished_scrobbles_qs.count()
    return data


def week_of_scrobbles(
    user=None, start=None, media: str = 'tracks'
) -> dict[str, int]:

    now = timezone.now()
    user_filter = Q()
    if user and user.is_authenticated:
        now = now_user_timezone(user.profile)
        user_filter = Q(user=user)

    if not start:
        start = datetime.combine(now.date(), datetime.min.time(), now.tzinfo)

    scrobble_day_dict = {}
    base_qs = Scrobble.objects.filter(user_filter, played_to_completion=True)

    media_filter = Q(track__isnull=False)
    if media == 'movies':
        media_filter = Q(video__video_type=Video.VideoType.MOVIE)
    if media == 'series':
        media_filter = Q(video__video_type=Video.VideoType.TV_EPISODE)

    for day in range(6, -1, -1):
        start_day = start - timedelta(days=day)
        end = datetime.combine(start_day, datetime.max.time(), now.tzinfo)
        day_of_week = start_day.strftime('%A')

        scrobble_day_dict[day_of_week] = base_qs.filter(
            media_filter,
            timestamp__gte=start_day,
            timestamp__lte=end,
            played_to_completion=True,
        ).count()

    return scrobble_day_dict


def top_tracks(
    user: "User", filter: str = "today", limit: int = 30
) -> List["Track"]:

    now = timezone.now()
    if user.is_authenticated:
        now = now_user_timezone(user.profile)

    start_of_today = datetime.combine(
        now.date(), datetime.min.time(), now.tzinfo
    )
    starting_day_of_current_week = now.date() - timedelta(
        days=now.today().isoweekday() % 7
    )
    starting_day_of_current_month = now.date().replace(day=1)
    starting_day_of_current_year = now.date().replace(month=1, day=1)

    time_filter = Q()
    if filter == "today":
        time_filter = Q(scrobble__timestamp__gte=start_of_today)
    if filter == "week":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_week)
    if filter == "month":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_month)
    if filter == "year":
        time_filter = Q(scrobble__timestamp__gte=starting_day_of_current_year)

    return (
        Track.objects.filter(time_filter)
        .annotate(
            num_scrobbles=Count(
                "scrobble",
                filter=Q(scrobble__played_to_completion=True),
                distinct=True,
            )
        )
        .order_by("-num_scrobbles")[:limit]
    )


def top_artists(
    user: "User", filter: str = "today", limit: int = 15
) -> List["Artist"]:
    time_filter = Q(track__scrobble__timestamp__gte=START_OF_TODAY)
    if filter == "week":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_WEEK
        )
    if filter == "month":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_MONTH
        )
    if filter == "year":
        time_filter = Q(
            track__scrobble__timestamp__gte=STARTING_DAY_OF_CURRENT_YEAR
        )

    return (
        Artist.objects.filter(time_filter)
        .annotate(
            num_scrobbles=Count(
                "track__scrobble",
                filter=Q(track__scrobble__played_to_completion=True),
                distinct=True,
            )
        )
        .order_by("-num_scrobbles")
    )


def artist_scrobble_count(artist_id: int, filter: str = "today") -> int:
    return Scrobble.objects.filter(track__artist=artist_id).count()
